@@ -1,4 +1,5 @@
-function (n, contrasts = FALSE) 
+contr.diff <-
+function(n)
 {
     if (is.numeric(n) && length(n) == 1) 
         levs <- 1:n
@@ -9,10 +10,6 @@ function (n, contrasts = FALSE)
     contr <- array(0, c(n, n), list(levs, levs))
     contr[col(contr) == row(contr)] <- 1
     contr[row(contr) - col(contr) == 1] <- -1
-    if (contrasts) {
-        if (n < 2) 
-            stop(paste("Contrasts not defined for", n - 1, "degrees of freedom"))
-        contr <- contr[, -1, drop = FALSE]
-    }
+    if (n < 2) stop(paste("Contrasts not defined for", n - 1, "degrees of freedom"))
     contr
 }
